@@ -14,7 +14,15 @@ using SPI.
 
 """
 
-__version__ = "2.3.16"
+try:
+    from typing import Optional
+    from circuitpython_typing import ReadableBuffer
+    from digitalio import DigitalInOut
+    from busio import SPI
+except ImportError:
+    pass
+
+__version__ = "2.3.18"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PN532.git"
 
 import time
@@ -28,7 +36,7 @@ _SPI_DATAREAD = const(0x03)
 _SPI_READY = const(0x01)
 
 
-def reverse_bit(num):
+def reverse_bit(num: int) -> int:
     """Turn an LSB byte to an MSB byte, and vice versa. Used for SPI as
     it is LSB for the PN532, but 99% of SPI implementations are MSB only!"""
     result = 0
@@ -44,13 +52,57 @@ class PN532_SPI(PN532):
     SPI device & chip select digitalInOut pin. Optional IRQ pin (not used),
     reset pin and debugging output."""
 
-    def __init__(self, spi, cs_pin, *, irq=None, reset=None, debug=False):
-        """Create an instance of the PN532 class using SPI"""
+    def __init__(
+        self,
+        spi: SPI,
+        cs_pin: DigitalInOut,
+        *,
+        irq: Optional[DigitalInOut] = None,
+        reset: Optional[DigitalInOut] = None,
+        debug: bool = False
+    ) -> None:
+        """Create an instance of the PN532 class using SPI
+        Optional IRQ pin (not used)
+
+        :param ~busio.SPI spi: The spi bus the PN532 is connected to.
+        :param digitalio.DigitalInOut cs: board pin the PN532 chip select line is connected to
+        :param digitalio.DigitalInOut irq: board pin the PN532 P32 is connected to
+        :param digitalio.DigitalInOut reset: board pin the PN532 RSTOUT_N is connected to
+        :param bool debug: if True print additional debug statements. Defaults to False
+
+
+        **Quickstart: Importing and using the device**
+        Here is an example of using the :class:`PN532_SPI` class.
+        First you will need to import the libraries to use the sensor
+
+        .. code-block:: python
+
+            import board
+            import busio
+            from digitalio import DigitalInOut
+            from adafruit_pn532.spi import PN532_SPI
+
+        Once this is done you can define your `busio.SPI` object and define your PN532 object
+
+        .. code-block:: python
+
+            spi = busio.SPI(board.SCK, board.MOSI, board.MISO)
+            cs_pin = DigitalInOut(board.D5)
+            pn532 = PN532_SPI(spi, cs_pin, debug=False)
+
+        Now you have access to the attributes and functions of the PN532 RFID/NFC
+        shield or breakout
+
+        .. code-block:: python
+
+            uid = pn532.read_passive_target(timeout=0.5)
+
+        """
         self.debug = debug
         self._spi = spi_device.SPIDevice(spi, cs_pin)
         super().__init__(debug=debug, irq=irq, reset=reset)
 
-    def _wakeup(self):
+    def _wakeup(self) -> None:
         """Send any special commands/data to wake up PN532"""
         if self._reset_pin:
             self._reset_pin.value = True
@@ -61,7 +113,7 @@ class PN532_SPI(PN532):
         self.low_power = False
         self.SAM_configuration()  # Put the PN532 back in normal mode
 
-    def _wait_ready(self, timeout=1):
+    def _wait_ready(self, timeout: float = 1) -> bool:
         """Poll PN532 if status byte is ready, up to `timeout` seconds"""
         status_cmd = bytearray([reverse_bit(_SPI_STATREAD), 0x00])
         status_response = bytearray([0x00, 0x00])
@@ -77,7 +129,7 @@ class PN532_SPI(PN532):
         # We timed out!
         return False
 
-    def _read_data(self, count):
+    def _read_data(self, count: int) -> bytearray:
         """Read a specified count of bytes from the PN532."""
         # Build a read request frame.
         frame = bytearray(count + 1)
@@ -92,7 +144,7 @@ class PN532_SPI(PN532):
             print("Reading: ", [hex(i) for i in frame[1:]])
         return frame[1:]
 
-    def _write_data(self, framebytes):
+    def _write_data(self, framebytes: ReadableBuffer) -> None:
         """Write a specified count of bytes to the PN532"""
         # start by making a frame with data write in front,
         # then rest of bytes, and LSBify it
